import { Table } from "../structures/Table";
import { DatabaseOptions } from "../typings/interfaces/DatabaseOptions";
import sqlite, { RunResult } from "better-sqlite3";
import { SqliteQueryOptions } from "../typings/interfaces/SqliteQueryOptions";
import { DataResolvable, InsertDataResolvable, TypedDataResolvable, TypedInsertDataResolvable } from "../typings/types/DataResolvable";
import { TypedEmitter } from "tiny-typed-emitter";
import { DatabaseEvents, SqliteSortOption, SqliteWhereOption } from "..";
import { Option } from "../typings";
export declare class Database<T = unknown> extends TypedEmitter<DatabaseEvents> {
    #private;
    options: DatabaseOptions<true>;
    tables: Map<string, Table>;
    wrapper: sqlite.Database;
    constructor(options?: DatabaseOptions);
    createTable<K extends keyof T>(name: K): Table;
    addTable(table: Table): Table;
    isReady(): boolean;
    private getTableRawColumns;
    private getRawTables;
    private updateTables;
    private resolveTable;
    insert(table: string, data: InsertDataResolvable): sqlite.RunResult;
    insert<K extends keyof T>(table: K | string, data: TypedDataResolvable<T[K]>): sqlite.RunResult;
    getRowCount(table: string, options?: SqliteQueryOptions | undefined): number;
    getRowCount<K extends keyof T>(table: K, options?: SqliteQueryOptions | undefined): number;
    insertMany<K extends keyof T>(table: K, data: TypedInsertDataResolvable<T[K]>[]): sqlite.RunResult;
    insertMany(table: string, data: InsertDataResolvable[]): sqlite.RunResult;
    has<K extends keyof T>(table: K, id: string): boolean;
    has(table: string, id: string): boolean;
    has(table: string, options: Exclude<string | SqliteQueryOptions["where"], undefined>): boolean;
    all(table: string, options?: SqliteQueryOptions | undefined): InsertDataResolvable[];
    all<K extends keyof T>(table: K, options?: SqliteQueryOptions | undefined): TypedInsertDataResolvable<T[K]>[];
    delete(table: string, options?: SqliteQueryOptions | undefined): RunResult;
    delete<K extends keyof T>(table: K, options?: SqliteQueryOptions | undefined): RunResult;
    update(table: string, data: DataResolvable, options: SqliteQueryOptions): RunResult;
    update<K extends keyof T>(table: K, data: TypedDataResolvable<T[K]>, options: SqliteQueryOptions): RunResult;
    set<K extends keyof T>(table: K, data: TypedDataResolvable<T[K]>, options: SqliteWhereOption | SqliteWhereOption[]): boolean;
    set(table: string, data: DataResolvable, options: SqliteWhereOption | SqliteWhereOption[]): boolean;
    upsert<K extends keyof T>(table: K, data: TypedInsertDataResolvable<T[K]>, options: SqliteWhereOption | SqliteWhereOption[]): boolean;
    upsert(table: string, data: InsertDataResolvable, options: SqliteWhereOption | SqliteWhereOption[]): boolean;
    getRowPosition<K extends keyof T>(table: K, sortOptions: SqliteSortOption, whereOptions: SqliteWhereOption | SqliteWhereOption[]): Option<number>;
    getRowPosition(table: string, sortOptions: SqliteSortOption, whereOptions: SqliteWhereOption | SqliteWhereOption[]): Option<number>;
    get<K extends keyof T>(table: K, id: string): TypedInsertDataResolvable<T[K]>;
    get<K extends keyof T>(table: K, options: Exclude<string | SqliteQueryOptions["where"], undefined>): TypedInsertDataResolvable<T[K]>;
    get(table: string, id: string): InsertDataResolvable;
    get(table: string, options: Exclude<string | SqliteQueryOptions["where"], undefined>): InsertDataResolvable;
    connect(): Promise<this>;
}
//# sourceMappingURL=Database.d.ts.map