"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _Database_ready;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Database = void 0;
const Table_1 = require("../structures/Table");
const DefaultDatabaseOptions_1 = require("../util/constants/DefaultDatabaseOptions");
const better_sqlite3_1 = __importDefault(require("better-sqlite3"));
const iterate_1 = __importDefault(require("../functions/iterate"));
const DatabaseError_1 = require("../structures/DatabaseError");
const DatabaseConnected_1 = __importDefault(require("../util/decorators/DatabaseConnected"));
const tiny_typed_emitter_1 = require("tiny-typed-emitter");
const cast_1 = __importDefault(require("../functions/cast"));
class Database extends tiny_typed_emitter_1.TypedEmitter {
    constructor(options = DefaultDatabaseOptions_1.DefaultDatabaseOptions) {
        super();
        this.tables = new Map();
        _Database_ready.set(this, false);
        this.options = options;
    }
    createTable(name) {
        const table = new Table_1.Table((0, cast_1.default)(name), this);
        this.tables.set(table.name, table);
        return table;
    }
    addTable(table) {
        this.tables.set(table.name, table["setDatabase"](this));
        return table;
    }
    isReady() {
        return __classPrivateFieldGet(this, _Database_ready, "f");
    }
    getTableRawColumns(name) {
        return this.wrapper.pragma(`table_info(${name})`);
    }
    getRawTables() {
        const tables = this.wrapper.prepare(`SELECT * FROM sqlite_schema WHERE type = 'table' AND name NOT LIKE 'sqlite_%'`).all();
        for (let i = 0, len = tables.length; i < len; i++) {
            const table = tables[i];
            table.columns = this.getTableRawColumns(table.name);
        }
        return tables;
    }
    updateTables() {
        const rawTables = this.getRawTables();
        (0, iterate_1.default)(this.tables.keys(), key => {
            const table = this.tables.get(key);
            const raw = rawTables.find(c => table.name === c.name);
            if (raw) {
                for (let i = 0, len = table.columns.length; i < len; i++) {
                    const column = table.columns[i];
                    let rawColumn = raw.columns.find(c => c.name === column.name);
                    if (!rawColumn) {
                        table.createColumn(column);
                    }
                    else {
                        raw.columns = raw.columns.filter(c => c.name !== rawColumn?.name);
                        continue;
                    }
                }
                if (raw.columns.length) {
                    table["drop"]();
                }
            }
            else {
                this.wrapper.prepare(`CREATE TABLE ${table.toCreationString()}`).run();
            }
        });
    }
    resolveTable(name) {
        const table = this.tables.get((0, cast_1.default)(name));
        if (!table)
            throw new DatabaseError_1.DatabaseError('TABLE_NOT_FOUND', name);
        return table;
    }
    insert(table, data) {
        return this.resolveTable(table).insert((0, cast_1.default)(data));
    }
    getRowCount(table, ...params) {
        return this.resolveTable(table).getRowCount(...params);
    }
    insertMany(table, data) {
        return this.resolveTable(table).insertMany((0, cast_1.default)(data));
    }
    has(table, type) {
        return this.resolveTable(table).has(type);
    }
    all(table, ...params) {
        return this.resolveTable(table).all(...params);
    }
    delete(table, ...params) {
        return this.resolveTable(table).delete(...params);
    }
    update(table, data, options) {
        return this.resolveTable(table).update(data, options);
    }
    set(table, data, options) {
        return this.resolveTable(table).set(data, options);
    }
    upsert(table, data, options) {
        return this.resolveTable(table).upsert((0, cast_1.default)(data), options);
    }
    getRowPosition(table, sortOptions, whereOptions) {
        return this.resolveTable(table).getRowPosition(sortOptions, whereOptions);
    }
    get(table, type) {
        return this.resolveTable(table).get(type);
    }
    connect() {
        return new Promise(async (resolve, reject) => {
            this.wrapper = new better_sqlite3_1.default(this.options.path);
            this.updateTables();
            __classPrivateFieldSet(this, _Database_ready, true, "f");
            this.emit("ready");
            resolve(this);
        });
    }
}
_Database_ready = new WeakMap();
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "insert", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "getRowCount", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "insertMany", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "has", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "all", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "delete", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "update", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "set", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "upsert", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "getRowPosition", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "get", null);
exports.Database = Database;
//# sourceMappingURL=Database.js.map